﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Configuration;
using System.ServiceModel;
using System.Diagnostics;

using EaseFilter.CommonObjects;

namespace EaseFilter.FolderLocker
{
    public partial class WebFormServices : Form
    {
        public static string serverURI = "http://www.encryptme.net/WebAPI.asmx";
        public WebFormServices()
        {
            InitializeComponent();
        }


        private static ServiceReference1.ServiceSoapClient GetServiceClient(ref string lastError)
        {
            ServiceReference1.ServiceSoapClient client = null;

            try
            {
                BasicHttpBinding binding = new BasicHttpBinding();
                binding.OpenTimeout = new TimeSpan(0, 0, 10);
                binding.CloseTimeout = new TimeSpan(0, 0, 10);
                binding.SendTimeout = new TimeSpan(0, 0, 10);
                binding.ReceiveTimeout = new TimeSpan(0, 0, 10);

                EndpointAddress endpoint = new EndpointAddress(new Uri(serverURI));

                client = new EaseFilter.FolderLocker.ServiceReference1.ServiceSoapClient(binding, endpoint);
            }
            catch (Exception ex)
            {
                lastError = "Connecting to server failed with error " + ex.Message; 
            }

            return client;
        }

     

        private void button_RegisterLicense_Click(object sender, EventArgs e)
        {
        }


        public static bool SignUpAccount(string accountName, string password, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.SignUpAccount(accountName, password, ref lastError);
            }
            catch (Exception ex)
            {
                lastError = "Sign in account failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool SignInAccount(string accountName, string password, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.SignInAccount(accountName, password, ref lastError);
            }
            catch (Exception ex)
            {
                lastError = "Sign in account failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }


        public static bool AddShareFile(string accountName, string password,string policy,ref long creationTime,ref string key,ref string iv, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.AddShareFile(accountName, password, policy, ref creationTime, ref key, ref iv, ref lastError);
            }
            catch (Exception ex)
            {
                lastError = "Add new file to server failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool GetSharedFilePermission( string userInfo, ref string key, ref string iv, ref uint accessFlags, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                retVal = client.GetSharedFilePermission(userInfo, ref key, ref iv,ref accessFlags, ref lastError);
            }
            catch (Exception ex)
            {
                lastError = "Get file key failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool GetFileDRInfo(string accountName, string password, string encryptionIV, ref string encryptedDRPolicy, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.GetSharedFileDRInfo(accountName, password, encryptionIV, ref encryptedDRPolicy, ref lastError);

            }
            catch (Exception ex)
            {
                lastError = "Get file DR info failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool ModifySharedFileDRInfo(string accountName, string password, string encryptedDRPolicy, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);             

                retVal = client.ModifySharedFileDRInfo(accountName, password, encryptedDRPolicy, ref lastError);

            }
            catch (Exception ex)
            {
                lastError = "Set file DR info failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool DeleteShareFile(string accountName, string password, string encryptionIV, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.DeleteSharedFile(accountName, password, encryptionIV, ref lastError);

            }
            catch (Exception ex)
            {
                lastError = "DeleteShareFile failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }


        public static bool GetFileList(string accountName, string password, ref string fileListStr, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.GetFileList(accountName, password, ref fileListStr, ref lastError);

            }
            catch (Exception ex)
            {
                lastError = "Get file list failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool GetAccessLog(string accountName, string password, ref string accessLog, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.GetAccessLog(accountName, password, ref accessLog, ref lastError);
            }
            catch (Exception ex)
            {
                lastError = "Get access log failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }

        public static bool ClearAccessLog(string accountName, string password, ref string lastError)
        {
            bool retVal = false;

            try
            {

                lastError = string.Empty;
                ServiceReference1.ServiceSoapClient client = GetServiceClient(ref lastError);

                if (null == client)
                {
                    return false;
                }

                accountName = FilterAPI.AESEncryptDecryptStr(accountName, FilterAPI.EncryptType.Encryption);
                password = FilterAPI.AESEncryptDecryptStr(password, FilterAPI.EncryptType.Encryption);

                retVal = client.ClearAccessLog(accountName, password, ref lastError);
            }
            catch (Exception ex)
            {
                lastError = "Clear access log failed with error:" + ex.Message;
                retVal = false;
            }

            return retVal;
        }
    }
}
