﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Technologies
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;
using System.Diagnostics;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using EaseFilter.CommonObjects;

namespace SecureSandbox
{
    public partial class SecureSandbox : Form
    {
        FilterRule selectedFilterRule = null;
        ProcessFilterRule selectedProcessFilterRule = null;
        RegistryFilterRule selectedRegistryFilterRule = null;

        //Purchase a license key with the link: http://www.easefilter.com/Order.htm
        //Email us to request a trial key: info@easefilter.com //free email is not accepted.
        string registerKey = GlobalConfig.registerKey;

        public SecureSandbox()
        {
            GlobalConfig.filterType = FilterAPI.FilterType.FILE_SYSTEM_CONTROL | FilterAPI.FilterType.FILE_SYSTEM_ENCRYPTION
                | FilterAPI.FilterType.FILE_SYSTEM_PROCESS | FilterAPI.FilterType.FILE_SYSTEM_REGISTRY;

            InitializeComponent();
            StartPosition = FormStartPosition.CenterScreen;
        
            InitListView();

            DisplayVersion();
        }

        ~SecureSandbox()
        {
            GlobalConfig.Stop();
        }

        private void SecureSandbox_FormClosed(object sender, FormClosedEventArgs e)
        {
            MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
            if (MessageBox.Show("Do you want to minimize to system tray?", "Close", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
            {

            }
            else
            {
                FilterAPI.ResetConfigData();
                FilterAPI.StopFilter();
                GlobalConfig.Stop();
                Application.Exit();
            }
        }


        private void DisplayVersion()
        {
            string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
            try
            {
                string filterDllPath = Path.Combine(GlobalConfig.AssemblyPath, "FilterAPI.Dll");
                version = FileVersionInfo.GetVersionInfo(filterDllPath).ProductVersion;
            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(43, "LoadFilterAPI Dll", EventLevel.Error, "FilterAPI.dll can't be found." + ex.Message);
            }

            this.Text += "    Version:  " + version;
        }

        void SetSelectedFilterRule(FilterRule filterRule)
        {
            selectedFilterRule = filterRule;
            selectedProcessFilterRule = GlobalConfig.GetProcessFilterRule("", selectedFilterRule.IncludeFileFilterMask);
            selectedRegistryFilterRule = GlobalConfig.GetRegistryFilterRule("", selectedFilterRule.IncludeFileFilterMask);
        }

        public void InitListView()
        {
            try
            {
                //init ListView control
                listView_Sandbox.Clear();		//clear control
                //create column header for ListView
                listView_Sandbox.Columns.Add("#", 20, System.Windows.Forms.HorizontalAlignment.Left);
                listView_Sandbox.Columns.Add("Sandbox Folder", 330, System.Windows.Forms.HorizontalAlignment.Left);
                listView_Sandbox.Columns.Add("Access Flag", 100, System.Windows.Forms.HorizontalAlignment.Left);

                foreach (FilterRule rule in GlobalConfig.FilterRules.Values)
                {
                    string[] itemStr = new string[listView_Sandbox.Columns.Count];
                    itemStr[0] = listView_Sandbox.Items.Count.ToString();
                    itemStr[1] = rule.IncludeFileFilterMask;
                    itemStr[2] = rule.AccessFlag.ToString();

                    ListViewItem item = new ListViewItem(itemStr, 0);
                    item.Tag = rule;
                    listView_Sandbox.Items.Add(item);

                    SetSelectedFilterRule(rule);
                }

                InitSandbox();
            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(127, "InitListView", EventLevel.Error, "InitListView failed with error:" + ex.Message);
            }
        }

        void InitNewFilterRule()
        {
            selectedFilterRule = new FilterRule();
            selectedProcessFilterRule = new ProcessFilterRule();
            selectedRegistryFilterRule = new RegistryFilterRule();

            //by default allow the binaries inside the sandbox to read/write the registry                
            selectedRegistryFilterRule.AccessFlag = FilterAPI.MAX_REGITRY_ACCESS_FLAG;

            selectedProcessFilterRule.ProcessId = selectedRegistryFilterRule.ProcessId = "";          

            //by default not allow the executable 
            selectedProcessFilterRule.ControlFlag = (uint)FilterAPI.ProcessControlFlag.DENY_NEW_PROCESS_CREATION;          

            //by default the sandbox folder doesn't allow being read/write by processes, if the processes want to access the sandbox, it needs to add process rights.
            selectedFilterRule.AccessFlag = (uint)(FilterAPI.ALLOW_MAX_RIGHT_ACCESS &
                ~(uint)(FilterAPI.ALLOW_FILE_READ_ACCESS | FilterAPI.ALLOW_FILE_CHANGE_ACCESS | (uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES));

            //set the maximum access rights to the sandbox for all binaries inside the sandbox 
            selectedProcessFilterRule.FileAccessRights = textBox_SandboxFolder.Text + "!" + FilterAPI.ALLOW_MAX_RIGHT_ACCESS.ToString() + ";";

            //allow the windows dll or exe to be read by the process, or it can't be loaded.
            selectedProcessFilterRule.FileAccessRights += "c:\\windows\\*!" + FilterAPI.ALLOW_FILE_READ_ACCESS + ";";

            //No access rights to all other folders by default.
            selectedProcessFilterRule.FileAccessRights += "*!" + ((uint)FilterAPI.AccessFlag.LEAST_ACCESS_FLAG).ToString() + ";";

            //allow processes from Windows to access the sandbox with read access rights
            selectedFilterRule.ProcessRights = "c:\\windows\\*!" + FilterAPI.ALLOW_FILE_READ_ACCESS.ToString() + ";";
        }

        private void InitSandbox()
        {
            try
            {
                if (selectedFilterRule == null)
                {
                    InitNewFilterRule();
                }
                else
                {
                    textBox_SandboxFolder.Text = selectedFilterRule.IncludeFileFilterMask;
                }

                uint fileAccessFlag = selectedFilterRule.AccessFlag;
                uint processControlFlag = selectedProcessFilterRule.ControlFlag;
                uint processRegistryControlFlag = selectedRegistryFilterRule.AccessFlag;               

                if ((processRegistryControlFlag & (uint)FilterAPI.ALLOW_READ_REGITRY_ACCESS_FLAG) != 0)
                {
                    checkBox_AllowReadRegistry.Checked = true;
                }
                else
                {
                    checkBox_AllowReadRegistry.Checked = false;
                }

                if ((processRegistryControlFlag & (uint)FilterAPI.ALLOW_CHANGE_REGITRY_ACCESS_FLAG) != 0)
                {
                    checkBox_AllowChangeRegistry.Checked = true;
                }
                else
                {
                    checkBox_AllowChangeRegistry.Checked = false;
                }

                if ((processControlFlag & (uint)FilterAPI.ProcessControlFlag.DENY_NEW_PROCESS_CREATION) != 0)
                {
                    checkBox_AllowExecute.Checked = false;
                }
                else
                {
                    checkBox_AllowExecute.Checked = true;
                }

                if ((fileAccessFlag & (uint)FilterAPI.ALLOW_FILE_READ_ACCESS) != 0)
                {
                    checkBox_AllowSandboxRead.Checked = true;
                }
                else
                {
                    checkBox_AllowSandboxRead.Checked = false;
                }

                if ((fileAccessFlag & (uint)FilterAPI.ALLOW_FILE_CHANGE_ACCESS) != 0)
                {
                    checkBox_AllowSandboxChange.Checked = true;
                }
                else
                {
                    checkBox_AllowSandboxChange.Checked = false;
                }

                if ((fileAccessFlag & (uint)FilterAPI.AccessFlag.ALLOW_DIRECTORY_LIST_ACCESS) != 0)
                {
                    checkBox_AllowFolderBrowsing.Checked = true;
                }
                else
                {
                    checkBox_AllowFolderBrowsing.Checked = false;
                }


                if ((fileAccessFlag & (uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES) != 0)
                {
                    checkBox_AllowReadEncryptedFiles.Checked = true;
                }
                else
                {
                    checkBox_AllowReadEncryptedFiles.Checked = false;
                }

                if ((fileAccessFlag & (uint)FilterAPI.AccessFlag.ENABLE_FILE_ENCRYPTION_RULE) != 0)
                {
                    checkBox_EnableEncryption.Checked = true;
                }
                else
                {
                    checkBox_EnableEncryption.Checked = false;
                }

                textBox_PassPhrase.Text = selectedFilterRule.EncryptionPassPhrase;
            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(254, "InitSandbox", EventLevel.Error, "InitSandbox failed with error:" + ex.Message);
            }

        }

        private bool GetSandboxSetting()
        {
            try
            {
                uint fileAccessFlag = selectedFilterRule.AccessFlag;
                uint processRegistryControlFlag = FilterAPI.MAX_REGITRY_ACCESS_FLAG;

                if (textBox_SandboxFolder.Text.Trim().Length == 0)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("The sandbox folder can't be empty.", "Add Filter Rule", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                selectedFilterRule.IncludeFileFilterMask = textBox_SandboxFolder.Text.Trim();
                selectedProcessFilterRule.ProcessNameFilterMask = textBox_SandboxFolder.Text.Trim();
                selectedRegistryFilterRule.ProcessNameFilterMask = textBox_SandboxFolder.Text.Trim();

                if (!textBox_SandboxFolder.Text.EndsWith("*"))
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("The sandbox folder must end with '*'.", "Add Filter Rule", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }

                if (checkBox_AllowReadRegistry.Checked)
                {
                    processRegistryControlFlag |= (uint)FilterAPI.ALLOW_READ_REGITRY_ACCESS_FLAG;
                }
                else
                {
                    processRegistryControlFlag &= ~(uint)FilterAPI.ALLOW_READ_REGITRY_ACCESS_FLAG;
                }

                if (checkBox_AllowChangeRegistry.Checked)
                {
                    processRegistryControlFlag |= (uint)FilterAPI.ALLOW_CHANGE_REGITRY_ACCESS_FLAG;
                }
                else
                {
                    processRegistryControlFlag &= ~(uint)FilterAPI.ALLOW_CHANGE_REGITRY_ACCESS_FLAG;
                }

                selectedRegistryFilterRule.AccessFlag = processRegistryControlFlag;

                if (checkBox_EnableEncryption.Checked)
                {
                    fileAccessFlag |= (uint)FilterAPI.AccessFlag.ENABLE_FILE_ENCRYPTION_RULE;

                    selectedFilterRule.EncryptMethod = (int)FilterAPI.EncryptionMethod.ENCRYPT_FILE_WITH_SAME_KEY_AND_UNIQUE_IV;
                    selectedFilterRule.EncryptionPassPhrase = textBox_PassPhrase.Text;

                    if (textBox_PassPhrase.Text.Trim().Length == 0)
                    {
                        MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                        MessageBox.Show("The encryption pass phrase can't be empty.", "apply Filter Rule", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return false;
                    }

                }

                selectedFilterRule.AccessFlag = fileAccessFlag;

                return true;
            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(326, "GetSandboxSetting", EventLevel.Error, "GetSandboxSetting failed with error:" + ex.Message);
                return false;
            }

        }

        private void button_AccessFlag_Click(object sender, EventArgs e)
        {
            if (selectedProcessFilterRule.ProcessNameFilterMask != textBox_SandboxFolder.Text)
            {
                InitNewFilterRule();
            }

            OptionForm optionForm = new OptionForm(OptionForm.OptionType.Access_Flag, selectedFilterRule.AccessFlag.ToString());

            if (optionForm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if (optionForm.AccessFlags > 0)
                {
                    selectedFilterRule.AccessFlag = optionForm.AccessFlags;
                }
            }

            InitSandbox();
        }
     

        private void button_SelectFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();

            if (folderBrowserDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                textBox_SandboxFolder.Text = folderBrowserDialog.SelectedPath + "\\*";
            }
        }

        private void button_ProcessFileAccessRights_Click(object sender, EventArgs e)
        {
            if (textBox_SandboxFolder.Text.Trim().Length == 0)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Sandbox folder can't be empty.", "config file access", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (!checkBox_AllowExecute.Checked)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("The binaries can't be executed, it can't access the files.", "config file access", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }

            if (selectedProcessFilterRule.ProcessNameFilterMask != textBox_SandboxFolder.Text)
            {
                InitNewFilterRule();
            }

            selectedProcessFilterRule.ProcessNameFilterMask = textBox_SandboxFolder.Text;          

            ProcessFileAccessRights processFileAccessRigths = new ProcessFileAccessRights(selectedProcessFilterRule);
            processFileAccessRigths.ShowDialog();
        }

        private void button_ConfigProcessRights_Click(object sender, EventArgs e)
        {
            if (textBox_SandboxFolder.Text.Trim().Length == 0)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("Sandbox folder can't be empty.", "config sendbox", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (selectedFilterRule.IncludeFileFilterMask != textBox_SandboxFolder.Text)
            {
                InitNewFilterRule();    
            }
            
            Form_ProcessRights processRights = new Form_ProcessRights(selectedFilterRule);
            processRights.ShowDialog();
            
        }

        private void listView_Sandbox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView_Sandbox.SelectedItems.Count == 0)
            {
                return;
            }

            SetSelectedFilterRule(((FilterRule)listView_Sandbox.SelectedItems[0].Tag).Copy());
          
            InitSandbox();
        }


        private void button_AddSandbox_Click(object sender, EventArgs e)
        {
            if (GetSandboxSetting())
            {
                GlobalConfig.AddFilterRule(selectedFilterRule);
                GlobalConfig.AddProcessFilterRule(selectedProcessFilterRule);
                GlobalConfig.AddRegistryFilterRule(selectedRegistryFilterRule);
                GlobalConfig.SaveConfigSetting();
            }

            InitListView();
        }

    
        private void button_DeleteSandbox_Click(object sender, EventArgs e)
        {
            if (listView_Sandbox.SelectedItems.Count == 0)
            {
                MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                MessageBox.Show("There are no sandbox selected.", "Delete sendbox", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            foreach (System.Windows.Forms.ListViewItem item in listView_Sandbox.SelectedItems)
            {
                FilterRule filterRule = (FilterRule)item.Tag;

                GlobalConfig.RemoveFilterRule(filterRule.IncludeFileFilterMask);

                ProcessFilterRule processFilterRule = GlobalConfig.GetProcessFilterRule("", filterRule.IncludeFileFilterMask);
                if (null != processFilterRule)
                {
                    GlobalConfig.RemoveProcessFilterRule(processFilterRule);
                }

                RegistryFilterRule registryFilterRule = GlobalConfig.GetRegistryFilterRule("", filterRule.IncludeFileFilterMask);
                if (null != registryFilterRule)
                {
                    GlobalConfig.RemoveRegistryFilterRule(registryFilterRule);
                }
                
                GlobalConfig.SaveConfigSetting();
            }

            if (GlobalConfig.FilterRules.Count > 0)
            {
                SetSelectedFilterRule(GlobalConfig.FilterRules.Values.ElementAt(0).Copy());
            }
            else
            {
                selectedFilterRule = null;
            }          

            InitListView();
        }

     

        private void checkBox_AllowSandboxRead_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_AllowSandboxRead.Checked)
            {
                selectedFilterRule.AccessFlag |= FilterAPI.ALLOW_FILE_READ_ACCESS;
            }
            else
            {
                selectedFilterRule.AccessFlag &= ~FilterAPI.ALLOW_FILE_READ_ACCESS;
            }
        }

        private void checkBox_AllowSandboxChange_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_AllowSandboxChange.Checked)
            {
                selectedFilterRule.AccessFlag |= FilterAPI.ALLOW_FILE_CHANGE_ACCESS;
            }
            else
            {
                selectedFilterRule.AccessFlag &= ~FilterAPI.ALLOW_FILE_CHANGE_ACCESS;
            }
        }

        private void checkBox_AllowFolderBrowsing_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_AllowFolderBrowsing.Checked)
            {
                selectedFilterRule.AccessFlag |= (uint)FilterAPI.AccessFlag.ALLOW_DIRECTORY_LIST_ACCESS;
            }
            else
            {
                selectedFilterRule.AccessFlag &= ~(uint)FilterAPI.AccessFlag.ALLOW_DIRECTORY_LIST_ACCESS;
            }
        }


        private void checkBox_AllowReadEncryptedFiles_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_AllowReadEncryptedFiles.Checked)
            {
                selectedFilterRule.AccessFlag |= (uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES;
            }
            else
            {
                selectedFilterRule.AccessFlag &= ~(uint)FilterAPI.AccessFlag.ALLOW_READ_ENCRYPTED_FILES;
            }
        }

        private void checkBox_EnableEncryption_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_EnableEncryption.Checked)
            {
                selectedFilterRule.AccessFlag |= (uint)FilterAPI.AccessFlag.ENABLE_FILE_ENCRYPTION_RULE;
                textBox_PassPhrase.ReadOnly = false;
            }
            else
            {
                selectedFilterRule.AccessFlag &= ~(uint)FilterAPI.AccessFlag.ENABLE_FILE_ENCRYPTION_RULE;
                textBox_PassPhrase.ReadOnly = true;
            }
        }

        private void checkBox_AllowExecute_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox_AllowExecute.Checked)
            {
                selectedProcessFilterRule.ControlFlag &= ~(uint)FilterAPI.ProcessControlFlag.DENY_NEW_PROCESS_CREATION;
            }
            else
            {
                selectedProcessFilterRule.ControlFlag |= (uint)FilterAPI.ProcessControlFlag.DENY_NEW_PROCESS_CREATION;
            }
        }

        private void textBox_SandboxFolder_Leave(object sender, EventArgs e)
        {
            if (selectedFilterRule.IncludeFileFilterMask != textBox_SandboxFolder.Text)
            {
                InitNewFilterRule();
            }
        }     


        Boolean FilterCallback(IntPtr sendDataPtr, IntPtr replyDataPtr)
        {
            Boolean ret = true;

            try
            {
                FilterAPI.MessageSendData messageSend = (FilterAPI.MessageSendData)Marshal.PtrToStructure(sendDataPtr, typeof(FilterAPI.MessageSendData));

                if (FilterAPI.MESSAGE_SEND_VERIFICATION_NUMBER != messageSend.VerificationNumber)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Received message corrupted.Please check if the MessageSendData structure is correct.");

                    EventManager.WriteMessage(139, "FilterCallback", EventLevel.Error, "Received message corrupted.Please check if the MessageSendData structure is correct.");
                    return false;
                }

                EventManager.WriteMessage(149, "FilterCallback", EventLevel.Verbose, "Received message Id#" + messageSend.MessageId + " type:" + messageSend.MessageType
                    + " CreateOptions:" + messageSend.CreateOptions.ToString("X") + " infoClass:" + messageSend.InfoClass + " fileName:" + messageSend.FileName);


                if (replyDataPtr.ToInt64() != 0)
                {
                    FilterAPI.MessageReplyData messageReply = (FilterAPI.MessageReplyData)Marshal.PtrToStructure(replyDataPtr, typeof(FilterAPI.MessageReplyData));

                    if (messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_REQUEST_ENCRYPTION_IV_AND_KEY)
                    {
                        //this is encryption filter rule with boolean config "REQUEST_ENCRYPT_KEY_AND_IV_FROM_SERVICE" enabled.                        
                        //the filter driver request the IV and key to open or create the encrypted file.                        

                        //if you want to deny the file open or creation, you set the value as below:
                        //messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_ACCESS_DENIED;
                        //messageReply.FilterStatus = (uint)FilterAPI.FilterStatus.FILTER_COMPLETE_PRE_OPERATION;

                        EventManager.WriteMessage(200, "filtercallback", EventLevel.Verbose, messageSend.FileName + " FILTER_REQUEST_ENCRYPTION_IV_AND_KEY");

                        //Here we return the test iv and key to the filter driver, you need to replace with you own iv and key in your code.
                        AESDataBuffer aesData = new AESDataBuffer();
                        aesData.AccessFlags = FilterAPI.ALLOW_MAX_RIGHT_ACCESS;
                        aesData.IV = FilterAPI.DEFAULT_IV_TAG;
                        aesData.IVLength = (uint)aesData.IV.Length;
                        aesData.EncryptionKey = Utils.GetKeyByPassPhrase(DigitalRightControl.PassPhrase);
                        aesData.EncryptionKeyLength = (uint)aesData.EncryptionKey.Length;

                        byte[] aesDataArray = DigitalRightControl.ConvertAESDataToByteArray(aesData);
                        messageReply.DataBufferLength = (uint)aesDataArray.Length;
                        Array.Copy(aesDataArray, messageReply.DataBuffer, aesDataArray.Length);

                        messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_SUCCESS;


                    }
                    else if (messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_REQUEST_ENCRYPTION_IV_AND_KEY_AND_TAGDATA)
                    {
                        //this is encryption filter rule with boolean config "REQUEST_ENCRYPT_KEY_IV_AND_TAGDATA_FROM_SERVICE" enabled.                        
                        //the filter driver request the IV and key to open or create the encrypted file.                        

                        //if you want to deny the file open or creation, you set the value as below:
                        //messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_ACCESS_DENIED;
                        //messageReply.FilterStatus = (uint)FilterAPI.FilterStatus.FILTER_COMPLETE_PRE_OPERATION;

                        if (messageSend.DataBufferLength > 0)
                        {
                            //this is the tag data which attached to the encrypted file .
                            string tagDataStr = Encoding.Unicode.GetString(messageSend.DataBuffer);
                            EventManager.WriteMessage(235, "filtercallback", EventLevel.Verbose, "EncryptedFile:" + messageSend.FileName + ",tagData:" + tagDataStr);
                        }

                        //Here we return the test iv ,key and tag data to the filter driver, you need to replace with you own iv and key in your code.
                        AESDataBuffer aesData = new AESDataBuffer();
                        aesData.AccessFlags = FilterAPI.ALLOW_MAX_RIGHT_ACCESS;
                        aesData.IV = FilterAPI.DEFAULT_IV_TAG;
                        aesData.IVLength = (uint)aesData.IV.Length;
                        aesData.EncryptionKey = Utils.GetKeyByPassPhrase(DigitalRightControl.PassPhrase);
                        aesData.EncryptionKeyLength = (uint)aesData.EncryptionKey.Length;
                        aesData.TagData = Encoding.Unicode.GetBytes("TagData:" + messageSend.FileName);
                        aesData.TagDataLength = (uint)aesData.TagData.Length;

                        byte[] aesDataArray = DigitalRightControl.ConvertAESDataToByteArray(aesData);
                        messageReply.DataBufferLength = (uint)aesDataArray.Length;
                        Array.Copy(aesDataArray, messageReply.DataBuffer, aesDataArray.Length);

                        messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_SUCCESS;

                    }
                    else
                    {
                        //this is for control filter driver when the pre-IO was registered.

                        //here you can control the IO behaviour and modify the data.
                        if (!FilterService.AuthorizeFileAccess(messageSend, ref messageReply))
                        {
                            //to comple the PRE_IO
                            messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_ACCESS_DENIED;
                            messageReply.FilterStatus = (uint)FilterAPI.FilterStatus.FILTER_COMPLETE_PRE_OPERATION;

                            EventManager.WriteMessage(160, "FilterCallback", EventLevel.Error, "Return error for I/O request:" + ((FilterAPI.MessageType)messageSend.MessageType).ToString() +
                                ",fileName:" + messageSend.FileName);
                        }
                        else
                        {

                            messageReply.MessageId = messageSend.MessageId;
                            messageReply.MessageType = messageSend.MessageType;
                            messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_SUCCESS;

                        }

                    }

                    Marshal.StructureToPtr(messageReply, replyDataPtr, true);
                }

                return ret;
            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(134, "FilterCallback", EventLevel.Error, "filter callback exception." + ex.Message);
                return false;
            }

        }

        void DisconnectCallback()
        {
            EventManager.WriteMessage(190, "DisconnectCallback", EventLevel.Information, "Filter Disconnected." + FilterAPI.GetLastErrorMessage());
        }
     

        private void toolStripButton_StartFilter_Click(object sender, EventArgs e)
        {
            try
            {
                string lastError = string.Empty;

                bool ret = FilterAPI.StartFilter((int)GlobalConfig.FilterConnectionThreads
                                            , registerKey
                                            , new FilterAPI.FilterDelegate(FilterCallback)
                                            , new FilterAPI.DisconnectDelegate(DisconnectCallback)
                                            , ref lastError);
                if (!ret)
                {
                    MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
                    MessageBox.Show("Start filter failed." + lastError);
                    return;
                }

                toolStripButton_StartFilter.Enabled = false;
                toolStripButton_Stop.Enabled = true;


                GlobalConfig.SendConfigSettingsToFilter();

                EventManager.WriteMessage(102, "StartFilter", EventLevel.Information, "Start filter service succeeded.");
            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(104, "StartFilter", EventLevel.Error, "Start filter service failed with error " + ex.Message);
            }
        }

        private void toolStripButton_Stop_Click(object sender, EventArgs e)
        {
            FilterAPI.ResetConfigData();
            FilterAPI.StopFilter();

            toolStripButton_StartFilter.Enabled = true;
            toolStripButton_Stop.Enabled = false;
        }

        private void toolStripButton_EventViewer_Click(object sender, EventArgs e)
        {
            EventForm.DisplayEventForm();
        }

        private void button_BinaryInfo_Click(object sender, EventArgs e)
        {
            string information = "Settings for the binaries inside the sandbox: allow/deny the binaries executing or allow/deny the registry access or changing,";
            information += "prevent the suspicious binaries(malware) from executing with this setting.\r\n\r\n";            
            information += "Restrict the file access rights: prevent the binaries insde the sandbox to read or change the files without your authorization.\r\n\r\n";
            information += "i.e. you can create a sandbox for your document setting folder,prevent the untrusted downloaded binaries from being launched there,";
            information += "or you can create a sandbox for your test software, only can read/write the files in the sandbox or other specific folder.\r\n\r\n";

            MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
            MessageBox.Show(information, "Executable binaries Setting", MessageBoxButtons.OK, MessageBoxIcon.Information);

        }

        private void button_FileInfo_Click(object sender, EventArgs e)
        {
            string information = "Set up the default file access rights to all processes, prevent the files being read or changed with the advanced settings.\r\n\r\n";
            information += "Authorize the specific processes access rights to read or change the files inside the sandbox.\r\n\r\n";
            information += "i.e. you create a sandbox for your confidential files,";
            information += "only allow authorized user/process to read or change the files there.\r\n\r\n";

            MessageBoxHelper.PrepToCenterMessageBoxOnForm(this);
            MessageBox.Show(information, "File access rights inside the sandbox", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

     

     
    }
}
